#pragma once
#include <cstdint>
#include <vector>

#include <lloiff.hpp>
#include <llosymbol_loader/llosymbol_loader_base.hpp>

namespace llo::s1
{
    /// <summary>
    /// base clase for all file format deconstructors...
    /// </summary>
    class dctor_base_t
    {
      protected:
        /// <summary>
        /// vector of bytes containing the raw image to be deconstructed...
        /// </summary>
        std::vector< std::uint8_t > img;

        /// <summary>
        /// symbol loader used when generating iff...
        /// </summary>
        llo::s1::symbol_loader_base_t *symbol_loader;

      public:
        /// <summary>
        /// set raw_img to the vector passed by reference...
        /// </summary>
        /// <param name="image">vector of bytes containing the raw image...</param>
        explicit dctor_base_t( const std::vector< std::uint8_t > &image,
                               llo::s1::symbol_loader_base_t *symbol_loader = nullptr )
            : img{ image }, symbol_loader{ symbol_loader }
        {
        }

        /// <summary>
        /// purely virtual method which gets overriden by classes that inherit
        /// dctor_base_t... fills the lloiff_t object full of data...
        /// </summary>
        /// <param name="iff"></param>
        /// <returns></returns>
        virtual void generate( lloiff_t &iff ) = 0;
    };
} // namespace llo::s1